inh_eqs_test = '''
    # Membrane voltage
    dv/dt = ( - I_leak - I_K - I_Na - I_SynE - I_SynExt - I_SynHipp - I_SynI + G_sin*I_exc + r*I_stim) / ((1*ufarad*cm**-2) * (size)) + noise : volt
    Vm = (- I_leak - I_K - I_Na) / ((1*ufarad*cm**-2) * (size))*tstep : volt


    # Channel activations (dimensionless)
    # From Wikipedia: n, m, and h are dimensionless quantities between 0 and 1 that are associated with potassium channel activation, sodium channel activation, and sodium channel inactivation, respectively
    dm/dt = (m_inf - m) / tau_m : 1
        m_inf = alpha_m / (alpha_m + beta_m) : 1
        tau_m = 0.2 / (alpha_m + beta_m) : second
    dn/dt = (n_inf - n) / tau_n : 1
        n_inf = alpha_n / (alpha_m + beta_n) : 1
        tau_n = 0.2 / (alpha_m + beta_n) : second
    dh/dt = (h_inf - h) / tau_h : 1
        h_inf = alpha_h / (alpha_h + beta_h) : 1
        tau_h = 0.2 / (alpha_h + beta_h) : second


    # Rate constants
    #alpha_m = 0.1/mV * (v + 35*mV) / (1 - exp(-0.1/mV * (v + 35*mV))) / ms : Hz
    alpha_m = 1. / exprel(-0.1/mV * (v + 35*mV)) / ms : Hz
    beta_m = 4. * exp( - (v + 60*mV) / (18*mV)) / ms : Hz

    #alpha_n = 0.01/mV * (v + 34*mV) / (1 - exp(-(0.1/mV) * (v + 34*mV))) / ms : Hz
    alpha_n = 0.1 / exprel(-(0.1/mV) * (v + 34*mV)) / ms : Hz
    beta_n = 0.125 * exp(-(v + 44*mV) / (80*mV)) / ms : Hz

    alpha_h = 0.07 * exp(-(v + 58*mV) / (20*mV)) / ms : Hz
    beta_h = 1 / (1 + exp(-0.1/mV * (v + 28*mV))) / ms : Hz


    # Currents
    I_leak = ((0.1e-3*siemens*cm**-2) * (size)) * (v + 65.*mV) : amp
    I_K = ((9e-3*siemens*cm**-2) * (size)) * (n**4) * (v + 90.*mV) : amp
    I_Na = ((35e-3*siemens*cm**-2) * (size)) * (m**3) * h * (v - (55.*mV)) : amp
    I_SynE = + ge * (v - (0*mV)) : amp
        dge/dt = (-ge+he) * (1. / (0.3*ms)) : siemens
        dhe/dt = -he/(5*ms) : siemens
    I_SynExt = + ge_ext * (v - (0*mV)) : amp
        dge_ext/dt = (-ge_ext+he_ext) * (1. / (0.3*ms)) : siemens
        dhe_ext/dt = -he_ext/(5*ms) : siemens
    I_SynHipp = + ge_hipp * (v - (0*mV)) : amp
        dge_hipp/dt = (-ge_hipp+he_hipp) * (1. / (0.3 * ms)) : siemens
        dhe_hipp/dt = -he_hipp/(5*ms) : siemens
    I_SynI = + gi * (v - (-80.*mV)) : amp
        dgi/dt = (-gi+hi) * (1. / (1*ms)) : siemens
        dhi/dt = -hi/(10*ms) : siemens


    # Noise term
    noise = sigma_noise_inh * (2 * (0.1e-3*siemens) / (1.*ufarad))**.5 * randn()/sqrt(tstep) : volt/second (constant over dt)


    # Parameters
    x_soma : metre
    y_soma : metre
    z_soma : metre
    G_sin = 1.*int(z_soma<100*scale)*int(z_soma>0*scale) : 1 # this is the mask/scaling for which neurons get the sinusoidal input
    I_exc : amp (linked) # same as in the pyCAN group, excitatory input from MS
    #I_exc = inp_theta(t) : amp
    r : 1
    I_stim = inputs_stim(t) : amp
    size : metre**2
'''
